//**************************************************************************
//  Copyright (c) 2007-2014  Daniel D Miller   home.comcast.net/~derelict
//  showfont.cpp - render DOS-era bit-mapped fonts as graphics glyphs
// 
//**************************************************************************
//  This is free software.  It is distributed under the terms of the
//  Code Project Open License (CPOL) V1.02 :
//  http://www.codeproject.com/info/cpol10.aspx
//
//  The main points subject to the terms of the License are:
//  
//  > Source Code and Executable Files can be used in commercial applications;
//  > Source Code and Executable Files can be redistributed; and
//  > Source Code can be modified to create derivative works.
//  > No claim of suitability, guarantee, or any warranty whatsoever is provided.
//    The software is provided "as-is".
//  > The Article accompanying the Work may not be distributed or republished 
//    without the Author's consent
//**************************************************************************

static char const * const Version = "ShowFont, Version 1.00" ;
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <sys/stat.h>
#include <time.h>

#include "resource.h"

//lint -e757  global declarator not referenced
//*********************************************************************
static HINSTANCE g_hinst = 0;

static COLORREF const set_bit   = 0xC0C0C0 ;
static COLORREF const clear_bit = 0x404040 ;
static COLORREF const bgnd      = 0x010101 ;

/****************************************************************************
 * This hook procedure, which allows ClearIcon to position the color dialog
 * as desired, was extracted from Nancy Cluts, Chapter 06, cmndlg32.
 * The technique of setting the dialog position by trapping WM_INITDIALOG
 * in the hook procedure, was taken from comments on the web.
****************************************************************************/
// #define  USE_CHOOSECOLOR_HOOK    1

#ifdef  USE_CHOOSECOLOR_HOOK
BOOL APIENTRY ChooseColorHookProc(HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
   switch (message) {
   //  list messages to be ignored
   case WM_CTLCOLORBTN:
   case WM_CTLCOLORSTATIC:
   case WM_CTLCOLOREDIT:
   case WM_MOUSEMOVE:
   case WM_NCMOUSEMOVE:
   case WM_NCHITTEST:
   case WM_SETCURSOR:
   case WM_TIMER:
   case WM_NOTIFY:
   case WM_COMMAND:  //  prints its own msgs below
      break;
   default:
      syslog("CC [%s]\n", lookup_winmsg_name(message)) ;
      break;
   }

   switch (message) {
   case WM_INITDIALOG:
      {
      DWORD UFLAGS = SWP_NOSIZE | SWP_NOZORDER | SWP_SHOWWINDOW;
      SetWindowPos(hDlg, HWND_TOP, 400, 300, 0, 0, UFLAGS);
      }   
      break;
        
   case WM_COMMAND:
      if (LOWORD(wParam) == IDOK) {
         syslog("IDOK\n") ;
         return (TRUE);
      }
      else if (LOWORD(wParam) == IDCANCEL) {
         syslog("IDCANCEL\n") ;
         return TRUE;
      } else {
         syslog("0x%04X\n", LOWORD(wParam)) ;
         
      }
      break;
   }
   return (FALSE);
}
#endif

//****************************************************************
static COLORREF select_color(HWND hwnd, COLORREF old_attr)
{
   static CHOOSECOLOR cc ;
   static COLORREF    crCustColors[16] ;

   // uint idx ;
   // for (idx=0; idx<16; idx++) {
   //    crCustColors[idx] = RGB(idx, idx, idx) ;
   // }

   ZeroMemory(&cc, sizeof(cc));
   cc.lStructSize    = sizeof (CHOOSECOLOR) ;
#ifdef  USE_CHOOSECOLOR_HOOK
   cc.rgbResult      = old_attr ;
   cc.lpCustColors   = crCustColors ;
   cc.Flags          = CC_RGBINIT | CC_FULLOPEN ;
   cc.Flags = CC_ENABLEHOOK;
   cc.lpfnHook = (LPCCHOOKPROC)ChooseColorHookProc;
   cc.lpTemplateName = (LPTSTR)NULL;
#else
   cc.rgbResult      = old_attr ;
   cc.lpCustColors   = crCustColors ;
   cc.Flags          = CC_RGBINIT | CC_FULLOPEN ;
   // cc.hwndOwner      = hwnd ; // this hangs parent, as well as me
#endif

   if (ChooseColor(&cc) == TRUE) {
      return cc.rgbResult ;
   } else {
      return 0 ;
   }
}  //lint !e715

//*********************************************************************
// Main Process CallBack Function
//*********************************************************************
static LRESULT APIENTRY MainProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   char msgstr[81] ;

   switch (msg) {
   // case WM_CREATE:
   case WM_INITDIALOG:
      wsprintfA(msgstr, "%s", Version) ;
      SetWindowTextA(hwnd, msgstr) ;

      SetClassLongA(hwnd, GCL_HICON,   (LONG) LoadIcon(g_hinst, (LPCSTR) SFICON));
      SetClassLongA(hwnd, GCL_HICONSM, (LONG) LoadIcon(g_hinst, (LPCSTR) SFICON));

      return true;

   case WM_COMMAND:  //  for keyboard handling
      {  //  create local context
      DWORD cmd    = HIWORD (wParam) ;
      DWORD target = LOWORD(wParam) ;

      switch (cmd) {
      case BN_CLICKED:
         switch(target) {
         case IDB_ATTR_SET:
            select_color(hwnd, set_bit) ;
            break;

         case IDB_ATTR_CLEAR:
            select_color(hwnd, clear_bit) ;
            break;

         case IDB_ATTR_BGND:
            select_color(hwnd, bgnd) ;
            break;
            
         case IDOK:
            PostMessage(hwnd, WM_CLOSE, 0, 0);
            break;

         default:
            return false;
         } //lint !e744  switch target
         return true;
      } //lint !e744  switch cmd
      break;
      }  //lint !e438 !e10  end local context

   //********************************************************************
   //  application shutdown handlers
   //********************************************************************
   case WM_CLOSE:
      DestroyWindow(hwnd);
      break;

   case WM_DESTROY:
      PostQuitMessage (0);
      break;

   default:
      return DefWindowProc (hwnd, msg, wParam, lParam);
	}
	return 0;
}

//***********************************************************************
//lint -esym(1784, WinMain)
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
   {
   g_hinst = hInstance;

   // HWND hwnd = 
   DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAIN_DIALOG), NULL, (DLGPROC) MainProc) ;
   return 0;

   // HWND hwnd = CreateDialog(g_hinst, MAKEINTRESOURCE(IDD_MAIN_DIALOG), NULL, (DLGPROC) MainProc) ;
   // if (hwnd == NULL) {
   //    syslog("CreateDialog: %s\n", get_system_message()) ;
   //    return 0;
   // }
   // 
   // MSG Msg;
   // while(GetMessage(&Msg, NULL,0,0)) {
   //    if(!IsDialogMessage(hwnd, &Msg)) {
   //        TranslateMessage(&Msg);
   //        DispatchMessage(&Msg);
   //    }
   // }
   // 
   // return (int) Msg.wParam ;
}  //lint !e715

